@@ -37,16 +37,19 @@
 #ifndef OPENIMAGEIO_IMAGECACHE_H
 #define OPENIMAGEIO_IMAGECACHE_H
 
-#include "OpenImageIO/ustring.h"
-#include "OpenImageIO/imageio.h"
+#include <OpenImageIO/ustring.h>
+#include <OpenImageIO/imageio.h>
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
+
+struct ROI;
 
 namespace pvt {
 // Forward declaration
 class ImageCacheImpl;
+class ImageCacheFile;
+class ImageCachePerThreadInfo;
 };
 
 
@@ -75,9 +78,8 @@ public:
     ImageCache (void) { }
     virtual ~ImageCache () { }
 
-    /// Close everything, free resources, start from scratch.
-    ///
-    virtual void clear () = 0;
+    OIIO_DEPRECATED("clear() was never implemented. Don't bother calling it. [1.7]")
+    virtual void clear () { }
 
     /// Set an attribute controlling the image cache.  Return true
     /// if the name and type were recognized and the attrib was set.
@@ -99,6 +101,8 @@ public:
     ///     string substitute_image : uses the named image in place of all
     ///                               texture and image references.
     ///     int unassociatedalpha : if nonzero, keep unassociated alpha images
+    ///     int max_errors_per_file : Limits how many errors to issue for
+    ///                               issue for each (default: 100)
     ///
     virtual bool attribute (string_view name, TypeDesc type,
                             const void *val) = 0;
@@ -108,15 +112,68 @@ public:
     virtual bool attribute (string_view name, double val) = 0;
     virtual bool attribute (string_view name, string_view val) = 0;
 
-    /// Get the named attribute, store it in value.
+    /// Get the named attribute, store it in *val. All of the attributes
+    /// that may be set with the attribute() call may also be queried with
+    /// getattribute().
+    ///
+    /// Additionally, there are some read-only attributes that can be
+    /// queried with getattribute():
+    ///     int total_files : the total number of unique files referenced by
+    ///             calls to the ImageCache.
+    ///     string[] all_filenames : an array that will be filled with the
+    ///             list of the names of all files referenced by calls to
+    ///             the ImageCache. (The array is of ustrings or char*'s.)
+    ///     stat:* : a variety of statistics (see full docs for details).
+    ///
     virtual bool getattribute (string_view name, TypeDesc type,
-                               void *val) = 0;
+                               void *val) const = 0;
     // Shortcuts for common types
-    virtual bool getattribute (string_view name, int &val) = 0;
-    virtual bool getattribute (string_view name, float &val) = 0;
-    virtual bool getattribute (string_view name, double &val) = 0;
-    virtual bool getattribute (string_view name, char **val) = 0;
-    virtual bool getattribute (string_view name, std::string &val) = 0;
+    virtual bool getattribute (string_view name, int &val) const = 0;
+    virtual bool getattribute (string_view name, float &val) const = 0;
+    virtual bool getattribute (string_view name, double &val) const = 0;
+    virtual bool getattribute (string_view name, char **val) const = 0;
+    virtual bool getattribute (string_view name, std::string &val) const = 0;
+
+    /// Define an opaque data type that allows us to have a pointer
+    /// to certain per-thread information that the ImageCache maintains.
+    /// Any given one of these should NEVER be shared between running
+    /// threads.
+    typedef pvt::ImageCachePerThreadInfo Perthread;
+
+    /// Retrieve a Perthread, unique to the calling thread. This is a
+    /// thread-specific pointer that will always return the Perthread for a
+    /// thread, which will also be automatically destroyed when the thread
+    /// terminates.
+    ///
+    /// Applications that want to manage their own Perthread pointers (with
+    /// create_thread_info and destroy_thread_info) should still call this,
+    /// but passing in their managed pointer. If the passed-in thread_info
+    /// is not NULL, it won't create a new one or retrieve a TSP, but it
+    /// will do other necessary housekeeping on the Perthread information.
+    virtual Perthread * get_perthread_info (Perthread *thread_info = NULL) = 0;
+
+    /// Create a new Perthread. It is the caller's responsibility to
+    /// eventually destroy it using destroy_thread_info().
+    virtual Perthread * create_thread_info () = 0;
+
+    /// Destroy a Perthread that was allocated by create_thread_info().
+    virtual void destroy_thread_info (Perthread *thread_info) = 0;
+
+    /// Define an opaque data type that allows us to have a handle to an
+    /// image (already having its name resolved) but without exposing
+    /// any internals.
+    typedef pvt::ImageCacheFile ImageHandle;
+
+    /// Retrieve an opaque handle for fast image lookups.  The opaque
+    /// pointer thread_info is thread-specific information returned by
+    /// get_perthread_info().  Return NULL if something has gone
+    /// horribly wrong.
+    virtual ImageHandle * get_image_handle (ustring filename,
+                                            Perthread *thread_info=NULL) = 0;
+
+    /// Return true if the image handle (previously returned by
+    /// get_image_handle()) is a valid image that can be subsequently read.
+    virtual bool good (ImageHandle *file) = 0;
 
     /// Given possibly-relative 'filename', resolve it using the search
     /// path rules and return the full resolved filename.
@@ -128,6 +185,9 @@ public:
     /// doesn't match the type requested. or some other failure.
     virtual bool get_image_info (ustring filename, int subimage, int miplevel,
                          ustring dataname, TypeDesc datatype, void *data) = 0;
+    virtual bool get_image_info (ImageHandle *file, Perthread *thread_info,
+                         int subimage, int miplevel,
+                         ustring dataname, TypeDesc datatype, void *data) = 0;
 
     /// Get the ImageSpec associated with the named image (the first
     /// subimage & miplevel by default, or as set by 'subimage' and
@@ -138,6 +198,10 @@ public:
     virtual bool get_imagespec (ustring filename, ImageSpec &spec,
                                 int subimage=0, int miplevel=0,
                                 bool native=false) = 0;
+    virtual bool get_imagespec (ImageHandle *file, Perthread *thread_info,
+                                ImageSpec &spec,
+                                int subimage=0, int miplevel=0,
+                                bool native=false) = 0;
 
     /// Return a pointer to an ImageSpec associated with the named image
     /// (the first subimage & miplevel by default, or as set by
@@ -152,6 +216,10 @@ public:
     /// file, or invalidate_all(), or destroys the ImageCache.
     virtual const ImageSpec *imagespec (ustring filename, int subimage=0,
                                         int miplevel=0, bool native=false) = 0;
+    virtual const ImageSpec *imagespec (ImageHandle *file,
+                                        Perthread *thread_info,
+                                        int subimage=0, int miplevel=0,
+                                        bool native=false) = 0;
 
     /// Retrieve the rectangle of pixels spanning [xbegin..xend) X
     /// [ybegin..yend) X [zbegin..zend), with "exclusive end" a la STL,
@@ -171,6 +239,11 @@ public:
                              int xbegin, int xend, int ybegin, int yend,
                              int zbegin, int zend,
                              TypeDesc format, void *result) = 0;
+    virtual bool get_pixels (ImageHandle *file, Perthread *thread_info,
+                             int subimage, int miplevel,
+                             int xbegin, int xend, int ybegin, int yend,
+                             int zbegin, int zend,
+                             TypeDesc format, void *result) = 0;
 
     /// Retrieve the rectangle of pixels spanning [xbegin..xend) X
     /// [ybegin..yend) X [zbegin..zend), channels [chbegin..chend), 
@@ -185,7 +258,9 @@ public:
     /// enough to accommodate the requested rectangle (taking into
     /// consideration its dimensions, number of channels, and data
     /// format).  Requested pixels outside the valid pixel data region
-    /// will be filled in with 0 values.
+    /// will be filled in with 0 values. The optional cache_chbegin and
+    /// cache_chend hint as to which range of channels should be cached
+    /// (which by default will be all channels of the file).
     ///
     /// Return true if the file is found and could be opened by an
     /// available ImageIO plugin, otherwise return false.
@@ -194,26 +269,48 @@ public:
                     int ybegin, int yend, int zbegin, int zend,
                     int chbegin, int chend, TypeDesc format, void *result,
                     stride_t xstride=AutoStride, stride_t ystride=AutoStride,
-                    stride_t zstride=AutoStride) = 0;
+                    stride_t zstride=AutoStride,
+                    int cache_chbegin = 0, int cache_chend = -1) = 0;
+    virtual bool get_pixels (ImageHandle *file, Perthread *thread_info,
+                    int subimage, int miplevel, int xbegin, int xend,
+                    int ybegin, int yend, int zbegin, int zend,
+                    int chbegin, int chend, TypeDesc format, void *result,
+                    stride_t xstride=AutoStride, stride_t ystride=AutoStride,
+                    stride_t zstride=AutoStride,
+                    int cache_chbegin = 0, int cache_chend = -1) = 0;
 
     /// Define an opaque data type that allows us to have a pointer
     /// to a tile but without exposing any internals.
     class Tile;
 
-    /// Find a tile given by an image filename, subimage & miplevel, and
-    /// pixel coordinates.  An opaque pointer to the tile will be
-    /// returned, or NULL if no such file (or tile within the file)
-    /// exists or can be read.  The tile will not be purged from the
-    /// cache until after release_tile() is called on the tile pointer
-    /// the same number of times that get_tile() was called (refcnt).
-    /// This is thread-safe!
+    /// Find a tile given by an image filename, subimage & miplevel, channel
+    /// range, and pixel coordinates.  An opaque pointer to the tile will be
+    /// returned, or NULL if no such file (or tile within the file) exists
+    /// or can be read.  The tile will not be purged from the cache until
+    /// after release_tile() is called on the tile pointer the same number
+    /// of times that get_tile() was called (refcnt). This is thread-safe!
+    /// If chend < chbegin, it will retrieve a tile containing all channels
+    /// in the file.
     virtual Tile * get_tile (ustring filename, int subimage, int miplevel,
-                                int x, int y, int z) = 0;
+                             int x, int y, int z,
+                             int chbegin = 0, int chend = -1) = 0;
+    virtual Tile * get_tile (ImageHandle *file, Perthread *thread_info,
+                             int subimage, int miplevel,
+                             int x, int y, int z,
+                             int chbegin = 0, int chend = -1) = 0;
 
     /// After finishing with a tile, release_tile will allow it to
     /// once again be purged from the tile cache if required.
     virtual void release_tile (Tile *tile) const = 0;
 
+    /// Retrieve the data type of the pixels stored in the tile, which may
+    /// be different than the type of the pixels in the disk file.
+    virtual TypeDesc tile_format (const Tile *tile) const = 0;
+
+    /// Retrieve the ROI describing the pixels and channels stored in the
+    /// tile.
+    virtual ROI tile_roi (const Tile *tile) const = 0;
+
     /// For a tile retrived by get_tile(), return a pointer to the
     /// pixel data itself, and also store in 'format' the data type that
     /// the pixels are internally stored in (which may be different than
@@ -250,14 +347,26 @@ public:
                            const ImageSpec *config=NULL) = 0;
 
     /// Preemptively add a tile corresponding to the named image, at the
-    /// given subimage and MIP level.  The tile added is the one whose
-    /// corner is (x,y,z), and buffer points to the pixels (in the given
-    /// format, with supplied strides) which will be copied and inserted
-    /// into the cache and made available for future lookups.
+    /// given subimage, MIP level, and channel range.  The tile added is the
+    /// one whose corner is (x,y,z), and buffer points to the pixels (in the
+    /// given format, with supplied strides) which will be copied and
+    /// inserted into the cache and made available for future lookups.
+    /// If chend < chbegin, it will add a tile containing the full set of
+    /// channels for the image.
+    virtual bool add_tile (ustring filename, int subimage, int miplevel,
+                     int x, int y, int z, int chbegin, int chend,
+                     TypeDesc format, const void *buffer,
+                     stride_t xstride=AutoStride, stride_t ystride=AutoStride,
+                     stride_t zstride=AutoStride) = 0;
+
+    OIIO_DEPRECATED("Use the version of add_tile with channel range. [1.6]")
     virtual bool add_tile (ustring filename, int subimage, int miplevel,
                      int x, int y, int z, TypeDesc format, const void *buffer,
                      stride_t xstride=AutoStride, stride_t ystride=AutoStride,
-                     stride_t zstride=AutoStride) = 0;
+                     stride_t zstride=AutoStride) {
+        return add_tile (filename, subimage, miplevel, x, y, z, 0, -1,
+                         format, buffer, xstride, ystride, zstride);
+    }
 
     /// If any of the API routines returned false indicating an error,
     /// this routine will return the error string (and clear any error
@@ -306,7 +415,6 @@ private:
 };
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
 #endif // OPENIMAGEIO_IMAGECACHE_H
