@@ -133,19 +133,19 @@
 #include <string>
 #include <iostream>
 #include <cstring>
-#include "export.h"
-#include "strutil.h"
-#include "dassert.h"
-#include "version.h"
+#include <OpenImageIO/export.h>
+#include <OpenImageIO/strutil.h>
+#include <OpenImageIO/string_view.h>
+#include <OpenImageIO/dassert.h>
+#include <OpenImageIO/oiioversion.h>
 
 #ifndef NULL
 #define NULL 0
 #endif
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
-class DLLPUBLIC ustring {
+class OIIO_API ustring {
 public:
     typedef char value_type;
     typedef value_type * pointer;
@@ -166,6 +166,13 @@ public:
         m_chars = str ? make_unique(str) : NULL;
     }
 
+    /// Construct a ustring from a string_view, which can be auto-converted
+    /// from either a null-terminated C string (char *) or a C++
+    /// std::string.
+    explicit ustring (string_view str) {
+        m_chars = str.data() ? make_unique(str) : NULL;
+    }
+
     /// Construct a ustring from at most n characters of str, starting at
     /// position pos.
     ustring (const char *str, size_type pos, size_type n)
@@ -174,21 +181,20 @@ public:
     /// Construct a ustring from the first n characters of str.
     ///
     ustring (const char *str, size_type n)
-        : m_chars (make_unique(std::string(str,n).c_str())) { }
+        : m_chars (make_unique(string_view(str,n))) { }
 
     /// Construct a ustring from n copies of character c.
     ///
     ustring (size_type n, char c)
         : m_chars (make_unique(std::string(n,c).c_str())) { }
 
-    /// Construct a ustring from a C++ std::string.
-    ///
-    explicit ustring (const std::string &str) { *this = ustring(str.c_str()); }
-
     /// Construct a ustring from an indexed substring of a std::string.
     ///
-    ustring (const std::string &str, size_type pos, size_type n=npos)
-        : m_chars (make_unique(std::string(str, pos, n).c_str())) { }
+    ustring (const std::string &str, size_type pos, size_type n=npos) {
+        string_view sref(str);
+        sref = sref.substr (pos, n);
+        m_chars = make_unique(sref);
+    }
 
     /// Copy construct a ustring from another ustring.
     ///
@@ -196,13 +202,19 @@ public:
 
     /// Construct a ustring from an indexed substring of a ustring.
     ///
-    ustring (const ustring &str, size_type pos, size_type n=npos)
-        : m_chars (make_unique(std::string(str.c_str(),pos,n).c_str())) { }
+    ustring (const ustring &str, size_type pos, size_type n=npos) {
+        string_view sref(str);
+        sref = sref.substr (pos, n);
+        m_chars = make_unique(sref);
+    }
 
     /// ustring destructor.
     ///
     ~ustring () { }
 
+    /// Conversion to string_view
+    operator string_view() const { return string_view(c_str(), length()); }
+
     /// Assign a ustring to *this.
     ///
     const ustring & assign (const ustring &str) {
@@ -245,6 +257,12 @@ public:
     const ustring & assign (size_type n, char c)
         { *this = ustring(n,c); return *this; }
 
+    /// Assign a string_view to *this.
+    const ustring & assign (string_view str) {
+        m_chars = str.length() ? make_unique(str) : NULL;
+        return *this;
+    }
+
     /// Assign a ustring to another ustring.
     ///
     const ustring & operator= (const ustring &str) { return assign(str); }
@@ -257,6 +275,10 @@ public:
     ///
     const ustring & operator= (const std::string &str) { return assign(str); }
 
+    /// Assign a string_view to a ustring.
+    ///
+    const ustring & operator= (string_view str) { return assign(str); }
+
     /// Assign a single char to a ustring.
     ///
     const ustring & operator= (char c) {
@@ -472,7 +494,7 @@ public:
     }
 
     size_type find_last_not_of(const char *s, size_type pos, size_type n) const {
-        return string().find(s, pos, n);
+        return string().find_last_not_of(s, pos, n);
     }
 
     size_type find_last_not_of(const char *s, size_type pos = npos) const {
@@ -499,6 +521,21 @@ public:
         return strcmp (c_str() ? c_str() : "", str.c_str());
     }
 
+    /// Return 0 if *this is lexicographically equal to str, -1 if
+    /// *this is lexicographically earlier than str, 1 if *this is
+    /// lexicographically after str.
+    int compare (string_view str) const {
+        return strncmp (c_str() ? c_str() : "",
+                        str.data() ? str.data() : "", str.length());
+    }
+
+    /// Return 0 if *this is lexicographically equal to str, -1 if
+    /// *this is lexicographically earlier than str, 1 if *this is
+    /// lexicographically after str.
+    int compare (const char *str) const {
+        return strcmp (c_str() ? c_str() : "", str ? str : "");
+    }
+
     /// Return 0 if a is lexicographically equal to b, -1 if a is
     /// lexicographically earlier than b, 1 if a is lexicographically
     /// after b.
@@ -511,7 +548,7 @@ public:
     /// this is a trivial pointer comparison, not a char-by-char loop as
     /// would be the case with a char* or a std::string.
     bool operator== (const ustring &str) const {
-      return c_str() == str.c_str();
+        return c_str() == str.c_str();
     }
 
     /// Test two ustrings for inequality -- are they comprised of different
@@ -519,29 +556,68 @@ public:
     /// this is a trivial pointer comparison, not a char-by-char loop as
     /// would be the case with a char* or a std::string.
     bool operator!= (const ustring &str) const {
-      return c_str() != str.c_str();
+        return c_str() != str.c_str();
     }
 
     /// Test a ustring (*this) for lexicographic equality with std::string
     /// x.
     bool operator== (const std::string &x) const { return compare(x) == 0; }
 
+    /// Test a ustring (*this) for lexicographic equality with string_view
+    /// x.
+    bool operator== (string_view x) const { return compare(x) == 0; }
+
+    /// Test a ustring (*this) for lexicographic equality with char* x.
+    bool operator== (const char *x) const { return compare(x) == 0; }
+
     /// Test for lexicographic equality between std::string a and ustring
     /// b.
     friend bool operator== (const std::string &a, const ustring &b) {
         return b.compare(a) == 0;
     }
 
+    /// Test for lexicographic equality between string_view a and ustring
+    /// b.
+    friend bool operator== (string_view a, const ustring &b) {
+        return b.compare(a) == 0;
+    }
+
+    /// Test for lexicographic equality between char* a and ustring
+    /// b.
+    friend bool operator== (const char *a, const ustring &b) {
+        return b.compare(a) == 0;
+    }
+
     /// Test a ustring (*this) for lexicographic inequality with
     /// std::string x.
     bool operator!= (const std::string &x) const { return compare(x) != 0; }
 
+    /// Test a ustring (*this) for lexicographic inequality with
+    /// string_view x.
+    bool operator!= (string_view x) const { return compare(x) != 0; }
+
+    /// Test a ustring (*this) for lexicographic inequality with
+    /// char* x.
+    bool operator!= (const char *x) const { return compare(x) != 0; }
+
     /// Test for lexicographic inequality between std::string a and
     /// ustring b.
     friend bool operator!= (const std::string &a, const ustring &b) {
         return b.compare(a) != 0;
     }
 
+    /// Test for lexicographic inequality between string_view a and
+    /// ustring b.
+    friend bool operator!= (string_view a, const ustring &b) {
+        return b.compare(a) != 0;
+    }
+
+    /// Test for lexicographic inequality between char* a and
+    /// ustring b.
+    friend bool operator!= (const char *a, const ustring &b) {
+        return b.compare(a) != 0;
+    }
+
     /// Test for lexicographic 'less', comes in handy for lots of STL
     /// containers and algorithms.
     bool operator< (const ustring &x) const { return compare(x) < 0; }
@@ -549,14 +625,18 @@ public:
     /// Construct a ustring in a printf-like fashion.  In other words,
     /// something like:
     ///    ustring s = ustring::format ("blah %d %g", (int)foo, (float)bar);
-    ///
-    static ustring format (const char *fmt, ...) OPENIMAGEIO_PRINTF_ARGS(1,2);
+    /// The argument list is fully typesafe.
+    template<typename... Args>
+    static ustring format (string_view fmt, const Args&... args)
+    {
+        return ustring (Strutil::format (fmt, args...));
+    }
 
     /// Generic stream output of a ustring.
     ///
     friend std::ostream & operator<< (std::ostream &out, const ustring &str) {
-        if (str.c_str())
-            out << str.c_str();
+        if (str.c_str() && out.good())
+            out.write (str.c_str(), str.size());
         return out;
     }
 
@@ -568,13 +648,13 @@ public:
     ///
     static size_t memory ();
 
-    /// Given a null-terminated string, return a pointer to the unique
+    /// Given a string_view, return a pointer to the unique
     /// version kept in the internal table (creating a new table entry
     /// if we haven't seen this sequence of characters before).  
     /// N.B.: this is equivalent to ustring(str).c_str().  It's also the
     /// routine that is used directly by ustring's internals to generate
     /// the canonical unique copy of the characters.
-    static const char * make_unique (const char *str);
+    static const char * make_unique (string_view str);
 
     /// Is this character pointer a unique ustring representation of
     /// those characters?  Useful for diagnostics and debugging.
@@ -610,7 +690,7 @@ public:
         size_t length;       // Length of the string; must be right before cap
         size_t dummy_capacity;  // Dummy field! must be right before refcount
         int    dummy_refcount;  // Dummy field! must be right before chars
-        TableRep (const char *s, size_t len);
+        TableRep (string_view strref, size_t hash);
         ~TableRep ();
         const char *c_str () const { return (const char *)(this + 1); }
     };
@@ -631,6 +711,33 @@ public:
 
 
 
+/// Functor class to use for comparisons when sorting ustrings, if you
+/// want the strings sorted lexicographically.
+class ustringLess
+{
+public:
+    size_t operator() (ustring a, ustring b) const {return a<b; }
+};
+
+
+/// Functor class to use for comparisons when sorting ustrings, if you
+/// don't care if the sort order is lexicographic. This sorts based on
+/// the pointers themselves, which is safe because once allocated, a
+/// ustring's characters will never be moved. But beware, the resulting
+/// sorting order may vary from run to run!
+class ustringPtrIsLess
+{
+public:
+    size_t operator() (ustring a, ustring b) const {
+        return size_t(a.data()) < size_t(b.data());
+    }
+};
+
+OIIO_DEPRECATED("Use ustringPtrIsLess [1.6]")
+typedef ustringPtrIsLess ustringHashIsLess;
+
+
+
 /// Case-insensitive comparison of ustrings.  For speed, this always
 /// uses a static locale that doesn't require a mutex lock.
 inline bool iequals (ustring a, ustring b) {
@@ -644,7 +751,6 @@ inline bool iequals (const std::string &a, ustring b) {
 }
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
 #endif // OPENIMAGEIO_USTRING_H
